/* CatalystX::CRUD::YUI custom css */

.xls_link {
    margin: 8px 0;
}

span.error, div.error
{
    font-size:95%;
    color:red;
    padding: 8px;
}

.red { color: red }

/* overall page layout */

body 
{
    text-align:left;
    /* font-size: 100%; */
}

.hidden
{
    display: none;
}

#main
{
    margin: 1em;
}

#help
{
    margin: 2em;
    width: 600px;
}

#help div
{
    margin-bottom: 8px;
}

#help h2,
#help h3
{
    font-weight: bold;
    margin-top: 1em;
    margin-bottom: 4px;
}

#help h2
{
    font-size: 130%;
}

#help h3
{
    font-size: 120%;
}

#help ul
{
    margin: 1em;
}

#help ul li
{
    list-style-type: disc; /* default */
    list-style-position: inside;
}

#help dl
{
    margin: 1em;
}

#help dl dt
{
    margin-left: 0.25em;
    margin-bottom: 0.25em;
    font-weight: bold;
    font-size: 110%;
}

#help dl dd
{
    margin-left: 1em;
    margin-bottom: 1em;
}

#results
{
    margin: 1em;
}

#head
{
    margin: 0;
    border: 1px solid #7a0019;
    background-color: #ffcc33;
}

#welcome
{
    float: left;
    padding: 6px 12px 6px 12px;
    font-size: 140%;
    font-weight: bold;
}

#quicksearch {
    float: right;
    margin: 2px;
    padding: 4px 16px 4px 16px;
    background-color: #7a0019;
}

#quicksearch input[type=text]
{
    font-family: Monaco, 'Andale Mono', fixed, monospace;
    padding: 2px;
    position: static;
}

.title
{
    font-weight: bold;
    font-size: 110%;
}

.center
{
    text-align: center;
}

/* UMN colors for links in forms */
#main a,
.crud a
{
    color: #7A0019;
    text-decoration: none;
}

a.box,
button.box
{
    color: #7A0019;
    text-decoration: none;
    padding: 4px;
    background-color: #fff;
    border: 1px solid #7A0019;
}

#main a:hover,
a.box:hover,
.crud a:hover,
button.box:hover
{
    background-color: #ffdd77;
    cursor:pointer;
}

div.related_object_matrix
{
    margin: 1em;
}

div.related_object_matrix caption
{
    background-color: #fff;
    border: 1px solid #ccc;
    padding: 4px;
}

#crud_footer
{
    clear: both;
    border-top: 1px solid #aaa;
    text-align: center;
    font-size: 90%;
    color: #7A0019;
}

/* results matrix remove 'button' */
div.yui-dt-col-_remove.yui-dt-liner
{
    text-align: center;
    color: #7A0019;
    font-size: 90%;
    cursor:pointer;
}

/* div.yui-dt-col-_remove:hover, */
div.yui-dt-col-_remove.hover
{
    color: #000;
    text-decoration: underline;
}

/* Resize Panel CSS */

.yui-panel-container .yui-resizepanel .bd {

    overflow: auto;
    background-color: #fff;

}

/*
    PLEASE NOTE: It is necessary to toggle the "overflow" property 
    of the body element between "hidden" and "auto" in order to 
    prevent the scrollbars from remaining visible after the the 
    ResizePanel is hidden.  For more information on this issue, 
    read the comments in the "container-core.css" file.
*/

.yui-panel-container.hide-scrollbars .yui-resizepanel .bd {

    overflow: hidden;

}

.yui-panel-container.show-scrollbars .yui-resizepanel .bd {

    overflow: auto;

}		


/*
    PLEASE NOTE: It is necessary to set the "overflow" property of
    the underlay element to "visible" in order for the 
    scrollbars on the body of a ResizePanel instance to be 
    visible.  By default the "overflow" property of the underlay 
    element is set to "auto" when a Panel is made visible on
    Gecko for Mac OS X to prevent scrollbars from poking through
    it on that browser + platform combintation.  For more 
    information on this issue, read the comments in the 
    "container-core.css" file.
*/

.yui-panel-container.show-scrollbars .underlay {

    overflow: visible;

}

.yui-resizepanel .resizehandle { 

     position: absolute; 
     width: 10px; 
     height: 10px; 
     right: 0;
     bottom: 0; 
     margin: 0; 
     padding: 0; 
     z-index: 1; 
     background: url(http://developer.yahoo.com/yui/examples/container/assets/img/corner_resize.gif) left bottom no-repeat;
     cursor: se-resize;

}

#panel_msg, .message {
    font-weight: bold;
    color: green;
}

#yui_history_iframe {
    position:absolute;
    top:0; left:0;
    width:1px; height:1px;
    visibility:hidden;
}
 
.temp_hiliter {
    background-color: #fff3b3;
}


/* tableless forms courtesy of http://bajooter.com/node/22  */

form.crud
{
    /* clear:both; */
    margin: 1em;
}

form.crud fieldset,
div.crud
{
    margin-top: 1em;
    padding: 1em;
    background-color: #eee;
    border:1px solid #aaa;
}


form.crud legend 
{
    padding: 0.2em 0.5em;
    background-color: #fff;
    border:1px solid #aaa;
    text-align:right;
}

form.crud label
{
    font-weight: bold;

}

form.crud div.wide input,
form.crud div.wide span.input
{
    float: none;
    clear:both;
    margin: 4px 100px;
    display: inline;
}
    
form.crud label, 
form.crud input,
form.crud span.input,
form.crud div.autocomplete_wrapper
{
    display: block;
    float: left;
    margin-bottom: 5px;
    margin-top:5px;
}

/* mac ff likes this */
form.crud label.autocomplete {
    margin-top:9px;
}

/* FF seems to like this, on mac anyway */
form.crud input.varchar,
form.crud input.char,
form.crud input.integer,
form.crud input.bigint,
form.crud input.int,
form.crud input.datetime,
form.crud input.date,
form.crud span.input.varchar
{
    margin-top: -1px;
}


form.crud legend 
{
font-weight:bold;
padding:5px;
}

form.crud select 
{
display: inline;
float:left;
margin-bottom: 5px;
margin-top: 5px;					
}


/* fieldset.narrow has narrower label column */
form.crud label
{
padding-right: 20px;				
text-align: right;
width: 170px;
}

form.crud fieldset fieldset label
{
    /* padding-right: 8px; */
    text-align: right;
    width: 140px;
}

form.crud fieldset.narrow label
{
padding-right: 20px;				
text-align: right;
width: 95px;
}

form.crud fieldset.wide label
{
padding-right: 20px;				
text-align: right;
width: 200px;
}

form.crud fieldset.narrow label.left
{
padding-right: 20px;				
text-align: left;
width: 95px;
}

form.crud br {
clear: left;
}

form.crud .submit 
{
display:inline;				
float:none;						          
margin-bottom:0px;

/* ie 5.x hack - fixes margin bug 
http://www.tantek.com/CSS/Examples/boxmodelhack.html*/
margin-left:95px;						
voice-family: "\"}\""; 
voice-family: inherit;

/* reset the margin back */
margin-left:115px;
margin-right:0px;
margin-top:10px;											
}

form.crud fieldset.list {
border:0;
float:left;
margin-bottom:3px;
}

/* in case someone adds a legend */
form.crud fieldset.list legend {
display:none;
}

form.crud fieldset.list label, 
form.crud fieldset.list input {
margin-bottom:2px;
margin-top:2px;
}

form.crud fieldset.list label {
margin-left: 5px;					
text-align:left;
}		


form.crud fieldset.inline
{
    display:inline;
    border:0;
    float:left;
    padding: 2px;
    margin: 2px 2px 2px 0px;
}
form.crud fieldset.inline label, 
form.crud fieldset.inline input {
    margin: 2px;
    padding: 2px;
    display:inline;
}

form.crud fieldset.inline label {
    margin: 2px 10px 2px 0px;
    padding: 2px;				
    text-align:left;
    display:inline;
    font-weight: normal;
    width:auto;
}

form.crud .inline
{
    display: inline;
    float:none;
    margin: 0;
    padding: 0;				
}

form.crud div.boolean_group,
form.crud div.boolean_group input,
form.crud div.boolean_group label
{
    clear:both;
    float:none;
    border:none;
    width:auto;
    display:inline;
}

form.crud div.boolean_group input,
form.crud div.boolean_group label
{
    font-weight:normal;
    padding-left: .5em;
    padding-right: 1em;
    margin: 0;
}

form.crud input[type=text],
form.crud input[type=password],
form.crud textarea
{
    font-family: Monaco, 'Andale Mono', fixed, monospace;
    padding: 2px;
}

/* buttons that look like links */
form.inline {
    display:inline;
    margin: 0;
    padding:0;
}

form.inline button {
    font-size: 12px;
}


/* YUI datatable overrides */
.yui-skin-sam .yui-dt-table caption {
    /* padding-bottom:1em; */
    padding: 0.5em;
    text-align:left;
    border: 1px solid #444;
    font-weight: bold;

}


.yui-skin-sam .yui-pg-container,
.yui-skin-sam .yui-dt-paginator {
    display:block;
    margin:6px 0;
    white-space:nowrap;
    padding: 4px;
}

.yui-skin-sam .yui-pg-first,
.yui-skin-sam .yui-pg-last,
.yui-skin-sam .yui-pg-current-page,
.yui-skin-sam .yui-dt-first,
.yui-skin-sam .yui-dt-paginator .yui-dt-last,
.yui-skin-sam .yui-dt-paginator .yui-dt-selected {
    padding:2px 6px;
}
.yui-skin-sam a.yui-pg-first,
.yui-skin-sam a.yui-pg-previous,
.yui-skin-sam a.yui-pg-next,
.yui-skin-sam a.yui-pg-last,
.yui-skin-sam a.yui-pg-page,
.yui-skin-sam .yui-dt-paginator a.yui-dt-first,
.yui-skin-sam .yui-dt-paginator a.yui-dt-last {
    text-decoration:none;
}
.yui-skin-sam .yui-dt-paginator .yui-dt-previous,
.yui-skin-sam .yui-dt-paginator .yui-dt-next {
    display:none;
}
.yui-skin-sam a.yui-pg-page,
.yui-skin-sam a.yui-dt-page {
    border:1px solid #7a0019;
    padding:2px 6px;
    text-decoration:none;
    background-color:#fff;
}
.yui-skin-sam .yui-pg-current-page,
.yui-skin-sam .yui-dt-selected {
    border:1px solid #000;
    /* background-color:#B2D2FF; */  /* YUI blue */
    background-color: #eee;
}
.yui-skin-sam .yui-pg-pages {
    margin-left:1ex;
    margin-right:1ex;
}
.yui-skin-sam .yui-pg-page {
    margin-right:1px;
    margin-left:1px;
}
.yui-skin-sam .yui-pg-first,
.yui-skin-sam .yui-pg-previous {
    margin-right:3px;
    border: 1px solid #aaa;
    padding: 2px 4px 2px 4px;
    background-color: #fff;
}
.yui-skin-sam .yui-pg-next,
.yui-skin-sam .yui-pg-last {
    margin-left:3px;
    border: 1px solid #aaa;
    padding: 2px 4px 2px 4px;
    background-color: #fff;
}

.yui-skin-sam a.yui-pg-first,
.yui-skin-sam a.yui-pg-previous,
.yui-skin-sam a.yui-pg-next,
.yui-skin-sam a.yui-pg-last
{
    border: 1px solid #7a0019;
    
}

.yui-skin-sam .yui-pg-current,
.yui-skin-sam .yui-pg-rpp-options {
    margin-right:1em;
    margin-left:1em;
    border: 1px solid #ccc;
    padding: 4px;
    background-color: #fff;
}

div.links {
    margin: 1em;
    padding: 1em;
    border: 1px solid #aaa;
    background-color: #eee;

}

div.links a {
    color: #7a0019;
    text-decoration:none;
    border: 1px solid #7a0019;
    background-color: #fff;
    padding: 4px;
}

div.links a:hover {
    background-color: #ffdd77;
}

div.links li {
    display: inline;
    list-style-type: none;
}

/* pagination */
.yui-skin-sam .pager_wrapper
{
    border: 1px solid #aaa;
    background-color: #fff;
    padding: 4px;
    margin-left: 1em;
    /* width: 80%; */
}

.yui-skin-sam .panel.pager_wrapper,
.yui-skin-sam .results.pager_wrapper
{
    margin-left: 0;
}

/* .yui-skin-sam .panel.pager_wrapper
{
    height: 36px;
}
 */
.yui-skin-sam .results.pager_wrapper
{
    margin-left: -1px;
    margin-bottom: 1em;
}

.pager_wrapper input.autocomplete
{
    position:static;
}

.yui-skin-sam .pager a
{
    color: #7a0019;
}

.panel.pager_wrapper .pager,
.results.pager_wrapper .pager
{
    float: left;
}

#panel_autocomplete,
#results_autocomplete
{
    border: 1px solid #aaa;
    background-color: #7a0019;
    padding: 4px;
    color: #fff;
    float: right;
    margin:4px 0;
}

.pager_wrapper br
{
    clear: both;
}

.crud .pager a.hover,
.crud .pager a:hover,
.yui-skin-sam a.yui-pg-page.hover,
.yui-skin-sam a.yui-pg-page:hover,
.yui-skin-sam a.yui-pg-first.hover,
.yui-skin-sam a.yui-pg-first:hover,
.yui-skin-sam a.yui-pg-previous.hover,
.yui-skin-sam a.yui-pg-previous:hover,
.yui-skin-sam a.yui-pg-next.hover,
.yui-skin-sam a.yui-pg-next:hover,
.yui-skin-sam a.yui-pg-last.hover,
.yui-skin-sam a.yui-pg-last:hover
{
    background-color: #ffdd77;
    color: #7a0019;
    border: 1px solid #7a0019;
}

/* undo crud effects */
#panel_autocomplete label
{
    font-weight:normal;
    text-align:left;
    width: auto;
    padding-right: 0;
    float:none;
    margin-top: 0;
    margin-bottom: 0;
    display: inline;

}

#panel_autocomplete input
{
    font-weight:normal;
    text-align:left;
    width: auto;
    padding-right: 0;
    float:none;
    margin-top: 0;
    margin-bottom: 0;
    display: inline;
}


/* sortable column headers should be colorful */
.yui-skin-sam .yui-dt th div a
{
    color: #7A0019;
}

.yui-skin-sam .yui-dt th div:hover
{
    color: #7A0019;
    background-color: #ffdd77;
}

/* popup panels */
#addListPanel.yui-panel .hd,
#links_panel_container .hd,
#daily_schedule_panel .hd
{
    background: #7a0019;
    text-align:center;
    border: 1px solid #ffdd77;
    color: #fff;
    font-size: 110%;
}

#addListPanel .bd,
#links_panel_container .bd,
#daily_schedule_panel .bd
{
    border: 1px solid #7a0019;
}

/* main menu */
.yui-skin-sam .bd {
    background-color: #7a0019;
    font-size: 105%;
}

.yui-skin-sam .yuimenubarlabel,
.yui-skin-sam .yuimenubaritemlabel,
.yui-skin-sam .yuimenubarnav 
{
    color: #fff;
}

/* highlighting */
.yui-skin-sam th.yui-dt-highlighted,
.yui-skin-sam th.yui-dt-highlighted a {
    /* background-color:#B2D2FF; */ /* med blue hover */
    background-color: #ffdd77;
    color: #7a0019;
    cursor:pointer;
}
.yui-skin-sam tr.yui-dt-highlighted,
.yui-skin-sam tr.yui-dt-highlighted td.yui-dt-asc,
.yui-skin-sam tr.yui-dt-highlighted td.yui-dt-desc,
.yui-skin-sam tr.yui-dt-even td.yui-dt-highlighted,
.yui-skin-sam tr.yui-dt-odd td.yui-dt-highlighted {
    cursor:pointer;
    /* background-color:#B2D2FF; */ /* med blue hover */
    background-color: #ffdd77;
    color: #7a0019;
}

.yui-skin-sam .yuimenubarnav .yuimenubaritemlabel-selected,
.yui-skin-sam .yuimenubarnav .yuimenuitemlabel-selected
{
    background-color: #ffcc33;
    color: #7a0019;
    cursor:pointer;
}

.yui-skin-sam .yui-dt-body { 
    cursor:pointer; /* when rows are selectable */
}

/* fix OSX gecko bug per http://developer.yahoo.com/yui/container/ */
.yui-panel-container.hide-scrollbars #addListPanel .yui-dt-bd,
.yui-panel-container.hide-scrollbars #links_panel_container .yui-dt-bd,
.yui-panel-container.hide-scrollbars #daily_schedule_panel .yui-dt-bd
{
    /* Hide scrollbars by default for Gecko on OS X */
    overflow: hidden;
}

.yui-panel-container.show-scrollbars #addListPanel .yui-dt-bd,
.yui-panel-container.show-scrollbars #links_panel_container .yui-dt-bd,
.yui-panel-container.show-scrollbars #daily_schedule_panel .yui-dt-bd
{
    /* Show scrollbars for Gecko on OS X when the Panel is visible  */
    overflow: auto;
}

/* pager popup */
form.crud div.pager_wrapper select,
#results select.yui-pg-rpp-options
{
    float: none;
    margin: 0;
    padding: 0;
}


/* tabview */

/* .yui-navset defaults to .yui-navset-top */
.yui-skin-sam .yui-navset .yui-nav,
.yui-skin-sam .yui-navset .yui-navset-top .yui-nav { /* protect nested tabviews from other orientations */
    border:solid #7A0019; /* color between tab list and content */
    border-width:0 0 5px;
    Xposition:relative;
    zoom:1;
}

.yui-skin-sam .yui-navset .yui-nav li,
.yui-skin-sam .yui-navset .yui-navset-top .yui-nav li {
    margin:0 0.16em 0 0; /* space between tabs */
    padding:1px 0 0; /* gecko: make room for overflow */
    zoom:1;
}

.yui-skin-sam .yui-navset .yui-nav .selected,
.yui-skin-sam .yui-navset .yui-navset-top .yui-nav .selected { 
    margin:0 0.16em -1px 0; /* for overlap */
}

.yui-skin-sam .yui-navset .yui-nav a,
.yui-skin-sam .yui-navset .yui-navset-top .yui-nav a {
    /*
    background:#d8d8d8 url(../../../../assets/skins/sam/sprite.png) repeat-x;
    */
    background: #ffffff;
    border:solid #7A0019;
    border-width:0 1px;
    color:#000;
    text-decoration:none;
}

.yui-skin-sam .yui-navset .yui-nav a em,
.yui-skin-sam .yui-navset .yui-navset-top .yui-nav a em {
    border:solid #7A0019;
    border-width:1px 0 0;
    cursor:hand;
    padding:0.25em .75em;
    left:0; right: 0; bottom: 0; /* protect from other orientations */
    top:-1px; /* for 1px rounded corners */
    position:relative;
}

.yui-skin-sam .yui-navset .yui-nav .selected a,
.yui-skin-sam .yui-navset .yui-nav .selected a:focus, /* no focus effect for selected */
.yui-skin-sam .yui-navset .yui-nav .selected a:hover { /* no hover effect for selected */
    /* background:#2647a0 url(../../../../assets/skins/sam/sprite.png) repeat-x left -1400px; *//* selected tab background */
    background: #ffdd77;
    color:#7A0019;
}

.yui-skin-sam .yui-navset .yui-nav a:hover,
.yui-skin-sam .yui-navset .yui-nav a:focus {
    /* background:#bfdaff url(../../../../assets/skins/sam/sprite.png) repeat-x left -1300px; *//* selected tab background */
    background: #ffdd77;
    outline:0;
}

.yui-skin-sam .yui-navset .yui-nav .selected a em {
    padding:0.35em 0.75em; /* raise selected tab */
}

.yui-skin-sam .yui-navset .yui-nav .selected a,
.yui-skin-sam .yui-navset .yui-nav .selected a em {
    border-color:#111111; /* selected tab border color */
}

.yui-skin-sam .yui-navset .yui-content {
    background:#ffffff; /* content background color */
}

.yui-skin-sam .yui-navset .yui-content,
.yui-skin-sam .yui-navset .yui-navset-top .yui-content {
    border:1px solid #7A0019; /* content border */
    border-top-color:#7A0019; /* different border color */
    padding:1em; /* content padding */
}

#relTabs .yui-content {
    height: 500px;  /* static height to try and reduce "jump" on clicking tab */
    overflow: auto;
}
